use 5.008;
use ExtUtils::MakeMaker;
use Cwd;
use File::Spec;
use ExtUtils::Depends;
use ExtUtils::PkgConfig;
use Gtk2::CodeGen;

# minimum required version of dependancies we need to build
our %build_reqs = (
	'perl-ExtUtils-Depends'   => '0.2',
	'perl-ExtUtils-PkgConfig' => '1.0',
	'perl-Glib'               => '1.04',
	'perl-Gtk2'               => '1.06',
);

# Writing a fake Makefile ensures that CPAN will pick up the correct
# dependencies and install them.
unless (eval "use ExtUtils::Depends;"
           . "use ExtUtils::PkgConfig;"
	   . "use Glib::MakeHelper;"
	   . "use Gtk2::CodeGen;"
           # just seeing if Glib is available isn't enough, make sure
           # it's recent enough, too
           . "use Glib '$build_reqs{'perl-Glib'}';"
           . "use Gtk2 '$build_reqs{'perl-Gtk2'}';"
           . "1") {
   warn "$@\n";
   WriteMakefile(
         PREREQ_FATAL => 1,
         PREREQ_PM    => {
            Glib::                => $build_reqs{'perl-Glib'},
            Gtk2::                => $build_reqs{'perl-Gtk2'},
            ExtUtils::Depends::   => $build_reqs{'perl-ExtUtils-Depends'},
            ExtUtils::PkgConfig:: => $build_reqs{'perl-ExtUtils-PkgConfig'},
         },
   );
   exit 1; # not reached
}

my %pkgcfg = ExtUtils::PkgConfig->find ('gtk+-2.0');

our @c_files = qw(gtksheet.c gtkitementry.c gtkextra.c
		  build/gtkextra-marshal.c build/gtksheetperl-gtypes.c);
our @xs_files = <xs/*.xs>;
our @headers = qw(gtksheet.h gtkitementry.h);

#
# generation...
#

mkdir 'build', 0777;

Gtk2::CodeGen->parse_maps ('gtksheet');
Gtk2::CodeGen->write_boot (xs_files => \@xs_files,
			   ignore => 'Gtk2::Ex::Sheet$');

system "glib-genmarshal --header --prefix=gtkextra gtkextra-marshal.list > build/gtkextra-marshal.h";
system "glib-genmarshal --body --prefix=gtkextra gtkextra-marshal.list > build/gtkextra-marshal.c";

system(q(glib-mkenums --fhead "#ifndef __GTKSHEETPERL_GTYPES_H__\n" ) .
                    q(--fhead "#define __GTKSHEETPERL_GTYPES_H__ 1\n\n" ) .
                    q(--fhead "#include <glib-object.h>\n\n" ) .
                    q(--fhead "G_BEGIN_DECLS\n\n" ) .
                    q(--eprod "#define GTKSHEETPERL_TYPE_@ENUMSHORT@ gtksheetperl_@enum_name@_get_type()\n" ) .
                    q(--eprod "GType gtksheetperl_@enum_name@_get_type (void);\n" ) .
                    q(--ftail "G_END_DECLS\n\n" ) .
                    q(--ftail "#endif /* __GTKSHEETPERL_GTYPES_H__ */\n" ) .
                    "@headers > build/gtksheetperl-gtypes.h");

system(q(glib-mkenums --fhead "#include <gtk/gtk.h>\n" ) .
                    q(--fhead "#include \"gtksheet.h\"\n" ) .
                    q(--vhead "static const G@Type@Value _gtksheetperl_@enum_name@_values[] = {" ) .
                    q(--vprod "  { @VALUENAME@, \"@VALUENAME@\", \"@valuenick@\" }," ) .
                    q(--vtail "  { 0, NULL, NULL }\n};\n\n" ) .
                    q(--vtail "GType\ngtksheetperl_@enum_name@_get_type (void)\n{\n" ) .
                    q(--vtail "  static GType type = 0;\n\n" ) .
                    q(--vtail "  if (!type)\n" ) .
                    q(--vtail "    type = g_@type@_register_static (\"@EnumName@\", _gtksheetperl_@enum_name@_values);\n\n" ) .
                    q(--vtail "  return type;\n}\n\n" ) .
                    "@headers > build/gtksheetperl-gtypes.c");

#
#
#

$gtksheet = ExtUtils::Depends->new ('Gtk2::Ex::Sheet', 'Gtk2');
$gtksheet->set_inc ($pkgcfg{cflags});
$gtksheet->set_libs ($pkgcfg{libs});
$gtksheet->add_c(@c_files);
$gtksheet->add_xs (@xs_files);
$gtksheet->add_pm ('Sheet.pm' => '$(INST_LIBDIR)/Sheet.pm');
my $cwd = cwd();
$gtksheet->add_typemaps (map {File::Spec->catfile($cwd,$_)}
                         'gtksheet.typemap', 'build/gtksheet.typemap');

# Don't install anything.
# $gtksheet->install (qw(gtksheet.h build/gtksheet-autogen.h));
#$gtksheet->install (qw(build/gtksheet-autogen.h));
#$gtksheet->save_config ('build/IFiles.pm');

WriteMakefile(
    NAME            => 'Gtk2::Ex::Sheet',
    VERSION_FROM    => 'Sheet.pm', # finds $VERSION
    ABSTRACT_FROM   => 'Sheet.pm', # retrieve abstract from module
    XSPROTOARG      => '-noprototypes',
    $gtksheet->get_makefile_vars,
#    OBJECT          => join(" ", map { s/\.(c|xs)$/\$(OBJ_EXT)/; "'$_'" } @xs_files, @c_files),
    PREREQ_PM       =>  {
    	Gtk2 => 1.06,
    },
);

package MY;
use Cwd;

sub postamble
{
	return Glib::MakeHelper->postamble_clean ()
# XXX
#	     . Glib::MakeHelper->postamble_docs (@main::xs_files)
	     . Glib::MakeHelper->postamble_rpms (
		'PERL_EXTUTILS_DEPENDS' =>
			$build_reqs{'perl-ExtUtils-Depends'},
		'PERL_EXTUTILS_PKGCONFIG' =>
			$build_reqs{'perl-ExtUtils-PkgConfig'},
		'PERL_GLIB' => $build_reqs{'perl-Glib'},
		'PERL_GTK' => $build_reqs{'perl-Gtk2'},
	       );
}
package main;
