#!/usr/bin/perl -w

use strict;
use Glib qw(TRUE FALSE);
use Gtk2 -init;
use Gtk2::Ex::Sheet;
use Gtk2::Gdk::Keysyms;
#include "pixmaps.h"

#define DEFAULT_PRECISION 3
#define DEFAULT_SPACE 8
#define NUM_SHEETS 3 

my $window;
my $notebook;
my @sheets;
my $location;
my $entry;
#GtkWidget *fgcolorcombo;
#GtkWidget *bgcolorcombo;
#GtkWidget *bordercombo;
#GdkPixmap *pixmap;
#GdkBitmap *mask;
#GtkWidget *bg_pixmap;
#GtkWidget *fg_pixmap;
my $left_button;
my $center_button;
my $right_button;
#GtkWidget *tpixmap;
#GtkWidget *smile;
my $curve;
my $popup;


sub current_sheet { $sheets[ $notebook->get_current_page ] }

sub popup_activated {
 my ($widget, $item) = @_;
 my $sheet = current_sheet();

 if ($item eq "Add Column") {
   $sheet->add_column (1);
 }

 if ($item eq "Add Row") {
   $sheet->add_row (1);
 }

 if ($item eq "Insert Row") {
   if ($sheet->state eq 'GTK_SHEET_ROW_SELECTED') {
     $sheet->insert_rows ($sheet->range->row0,                       
                          $sheet->range->rowi - $sheet->range->row0 + 1);
   }
 }

 if ($item eq "Insert Column") {
   if ($sheet->state eq 'GTK_SHEET_COLUMN_SELECTED') {
     $sheet->insert_columns ($sheet->range->col0,                       
                             $sheet->range->coli - $sheet->range->col0 + 1);
   }
 } 

 if ($item eq "Delete Row") {
   if ($sheet->state eq 'GTK_SHEET_ROW_SELECTED') {
     $sheet->delete_rows ($sheet->range->row0,
                          $sheet->range->rowi - $sheet->range->row0 + 1);
   }
 }

 if ($item eq "Delete Column") {
   if ($sheet->state eq 'GTK_SHEET_COLUMN_SELECTED') {
     $sheet->delete_columns ($sheet->range->col0,
                             $sheet->range->coli - $sheet->range->col0 + 1);
   }
 } 

 if ($item, "Clear Cells") {
   if ($sheet->state ne 'GTK_SHEET_NORMAL') {
     $sheet->range_clear ($sheet->range);
   }
 } 

 $popup->destroy;
 return TRUE;
}

sub build_menu {
	my $sheet = shift;

	my @items = (
		{ label => "Add Column",    state => undef },
		{ label => "Add Row",       state => undef }, 
		{ label => "Insert Row",    state => 'GTK_SHEET_ROW_SELECTED' }, 
		{ label => "Insert Column", state => 'GTK_SHEET_COLUMN_SELECTED' }, 
		{ label => "Delete Row",    state => 'GTK_SHEET_ROW_SELECTED' }, 
                { label => "Delete Column", state => 'GTK_SHEET_ROW_SELECTED' }, 
		{ label => "Clear Cells",   state => 'GTK_SHEET_COLUMN_SELECTED' }, 
	);

	my $menu = Gtk2::Menu->new;

	foreach my $i (@items) {
	 	my $item = Gtk2::MenuItem->new ($i->{label});

	 	$item->signal_connect (activate => \&popup_activated, $i->{label});

                $item->set_sensitive (FALSE)
			if $i->{state} and
			   $i->{state} ne $sheet->state;

		$item->show;
	    	$menu->append ($item);
	}

	return $menu;
}

sub do_popup{
   my ($widget, $event, $sheet) = @_;

   my (undef, undef, $mods) = $sheet->window->get_pointer;
   if ($mods >= 'button3-mask') {
    $popup = build_menu ($sheet);

    $popup->popup (undef, undef, undef, undef,
		   $event->button, $event->time);
   }

   return FALSE;
}

#sub format_text {
#  my ($sheet, $text) = @_;
#  , gint *justification, char *label)
#
#  double auxval;
#  int digspace=0;
#  int cell_width, char_width;
#  double val = 0.0;
#  int format;
#  double space;
#  int intspace;
#  int nonzero=FALSE;
#  int ipos;
#  char nchar;
#  enum {EMPTY, TEXT, NUMERIC};
#
#  cell_width=sheet->column[sheet->active_cell.col].width;
#  char_width = gdk_char_width (GTK_WIDGET(sheet)->style->private_font,(gchar)'X');
#  space= (double)cell_width/(double)char_width;
#
#  intspace=MIN(space, DEFAULT_SPACE);
#
#  format=EMPTY;
#  if(strlen(text) != 0)
#  { 
#        for(ipos=0; ipos<strlen(text); ipos++){
#
#          switch(nchar=text[ipos]){
#           case '.':
#           case ' ': case ',':
#           case '-': case '+':
#           case 'd': case 'D':
#           case 'E': case 'e':
#           case '1': case '2': case '3': case '4': case '5': case '6':
#           case '7': case '8': case '9':
#            nonzero=TRUE;
#            break;
#           case '0':
#            break;
#           default:
#            format=TEXT;
#          }
#          if(format != EMPTY) break;
#        }
#        val=atof(text);
#        if(format!=EMPTY || (val==0. && nonzero))
#             format = TEXT;
#        else
#             format = NUMERIC;
#  }
#
#  switch(format){
#    case TEXT:
#    case EMPTY:
#      strcpy(label, text);
#      return;
#    case NUMERIC:
#      val=atof(text);
#      *justification = GTK_JUSTIFY_RIGHT;
#  }
#
#  auxval= val < 0 ? -val : val;
#
#  while(auxval<1 && auxval != 0.){
#   auxval=auxval*10.;
#   digspace+=1;
#  }
#
#  if(digspace+DEFAULT_PRECISION+1>intspace || digspace > DEFAULT_PRECISION){
#    sprintf (label, "%*.*E", intspace, DEFAULT_PRECISION, val);
#  }
#  else
#    {
#    intspace=MIN(intspace, strlen(text)-digspace-1);
#    sprintf (label, "%*.*f", intspace, DEFAULT_PRECISION, val);
#    if(strlen(label) > space)
#      sprintf (label, "%*.*E", intspace, DEFAULT_PRECISION, val);
#  }
#}

#void
#parse_numbers(GtkWidget *widget, gpointer data)
#{
# GtkSheet *sheet;
# char label[400];
# gint justification;
# GtkSheetCellAttr attr;
#
# sheet=GTK_SHEET(widget);
#
# gtk_sheet_get_attributes(sheet, sheet->active_cell.row,
#			         sheet->active_cell.col,
#                                 &attr); 
#
# justification = attr.justification; 
# format_text(sheet, gtk_entry_get_text(GTK_ENTRY(sheet->sheet_entry)), 
#             &justification, label);
#
# $sheet->set_cell (sheet->active_cell.row,
#			   sheet->active_cell.col,
#                           justification, label); 
#
#}

sub clipboard_handler {
  my ($sheet, $key) = @_;

  if ($key->state & 'GDK_CONTROL_MASK' ||
      $key->keyval == $Gtk2::Gdk::Keysyms{Control_L} ||
      $key->keyval == $Gtk2::Gdk::Keysyms{Control_R}) {
    if (($key->keyval=='c' || $key->keyval == 'C') &&
         $sheet->state != 'GTK_STATE_NORMAL') {
	    $sheet->unclip_range if $sheet->in_clip;
            $sheet->clip_range ($sheet->range);
#            gtk_sheet_unselect_range(sheet);
    }
    if ($key->keyval=='x' || $key->keyval == 'X') {
            $sheet->unclip_range;
    }
  }

  return FALSE;
}

sub resize_handler {
  my ($widget, $old_range, $new_range) = @_;
  printf "OLD SELECTION: %d %d %d %d\n",
  	 $old_range->row0, $old_range->col0,
	 $old_range->rowi, $old_range->coli;
  printf "NEW SELECTION: %d %d %d %d\n",
  	 $new_range->row0, $new_range->col0,
	 $new_range->rowi, $new_range->coli;
}

sub move_handler {
  my ($widget, $old_range, $new_range) = @_;
  printf "OLD SELECTION: %d %d %d %d\n",
  	 $old_range->row0, $old_range->col0,
	 $old_range->rowi, $old_range->coli;
  printf "NEW SELECTION: %d %d %d %d\n",
  	 $new_range->row0, $new_range->col0,
	 $new_range->rowi, $new_range->coli;
}

sub change_entry {
  my ($sheet, $row, $col) = @_;
#  , gint *new_row, gint *new_col,
#             gpointer data)
  my $new_col = \2;

  if ($$new_col == 0 && ($col != 0 || $sheet->state ne 'GTK_STATE_NORMAL')){
         $sheet->change_entry (gtk_combo_get_type());
  }

  if ($$new_col == 1 && ($col != 1 || $sheet->state ne 'GTK_STATE_NORMAL')) {
         $sheet->change_entry ('Gtk2::Entry');
  }

  if ($$new_col == 2 && ($col != 2 || $sheet->state ne 'GTK_STATE_NORMAL')) {
         $sheet->change_entry ('Gtk2::SpinButton');
  }

  if ($$new_col >= 3 && ($col < 3 || $sheet->state ne 'GTK_STATE_NORMAL')) {
         $sheet->change_entry ('Gtk2::Ex::Sheet::ItemEntry');
  }

  return TRUE;
}


sub alarm_change {
 my ($widget, $row, $col) = @_;
 printf("CHANGE CELL: %d %d\n",$row,$col);
}

sub alarm_activate {
 my ($widget, $row, $col) = @_;

 printf("ACTIVATE CELL: %d %d\n",$row,$col);

# range.row0 = range.rowi = row;
# range.col0 = range.coli = col;
# gtk_sheet_range_set_justification(GTK_SHEET(widget), &range, GTK_JUSTIFY_LEFT);
 return TRUE;
}

sub alarm_deactivate {
 my ($widget, $row, $col) = @_;

 printf("DEACTIVATE CELL: %d %d\n",$row,$col);

# range.row0 = range.rowi = row;
# range.col0 = range.coli = col;
# gtk_sheet_range_set_justification(GTK_SHEET(widget), &range, GTK_JUSTIFY_RIGHT);
 return TRUE;
}

sub alarm_traverse {
 my ($widget, $row, $col, $new_row, $new_col) = @_;
 printf("TRAVERSE: %d %d %d %d\n", $row, $col, $new_row, $new_col);
 return TRUE;
}

sub show_child {
 if (!$curve->mapped) {
	 $sheets[0]->attach_floating ($curve, 2, 7);
#         gtk_sheet_put(GTK_SHEET(sheets[0]), curve, 550, 120);
 }
}

sub build_example1 {
 my $sheet = shift;
 my $font_name1 = "Arial 36";
 my $font_name2 = "Arial 28";

# my $colormap = Gtk2::Gdk::Colormap->get_system;
 my $colormap = $sheet->get_colormap;

 my $color = Gtk2::Gdk::Color->parse ("light yellow");
 $colormap->color_alloc ($color);
 $sheet->set_background ($color);
 $color = Gtk2::Gdk::Color->parse ("light blue");
 $colormap->color_alloc ($color);
 $sheet->set_grid ($color);

 foreach my $i (0..$sheet->maxcol) {
   my $name = chr (ord ('A') + $i);
   $sheet->column_button_add_label ($i, $name);
   $sheet->set_column_title ($i, $name);
 }

 $sheet->row_button_add_label (0, "This is\na multiline\nlabel");
 $sheet->row_button_add_label (1, "This is long label");
# gtk_sheet_column_button_add_label(sheet, 7, "This is\na multiline\nlabel");
# gtk_sheet_column_button_add_label(sheet, 8, "This is long label");
 $sheet->row_button_justify (0, 'right');

 my $range = Gtk2::Ex::Sheet::Range->new (1, 1, 2, 3);

 $sheet->clip_range ($range);

 my $font = Gtk2::Pango::FontDescription->from_string ($font_name2);
 $sheet->range_set_font ($range, $font);

 $color = Gtk2::Gdk::Color->parse ("red");
 $colormap->color_alloc ($color);
 $sheet->range-set_foreground ($range, $color);

 $sheet->set_cell (1, 2, 'GTK_JUSTIFY_CENTER', "Welcome to");

 $range->row0 (2);

 $font = Gtk2::Pango::FontDescription->from_string ($font_name1);
 $sheet->range_set_font(sheet, $range, $font);

 $color = Gtk2::Gdk::Color->parse ("blue");
 $colormap->color_alloc ($color);
 $sheet->range_set_foreground ($range, $color);

 $sheet->set_cell (2,2, 'GTK_JUSTIFY_CENTER', "GtkSheet");

 $range->row0 (3);
 $range->rowi (3);
 $range->col0 (0);
 $range->coli (4);
 $color = Gtk2::Gdk::Color->parse ("dark gray");
 $colormap->color_alloc ($color);
 $sheet->range_set_background ($range, $color);
 $color = Gtk2::Gdk::Color->parse ("green");
 $colormap->color_alloc ($color);
 $sheet->range_set_foreground ($range, $color);

 $sheet->set_cell (3,2,'center', "a Matrix widget for Gtk+");

 $sheet->set_cell (4,1,'left', "GtkSheet is a matrix where you can allocate cells of text.");
 $sheet->set_cell (5,1,'left', "Cell contents can be edited interactively with an specially designed entry");
 $sheet->set_cell (6,1,'left', "You can change colors, borders, and many other attributes");
 $sheet->set_cell (7,1, 'left', "Drag & drop or resize the selection clicking the corner or border");
 $sheet->set_cell (8, 1, 'left', "Store the selection on the clipboard pressing Ctrl-C");
 $sheet->set_cell (9, 1, 'left', "(The selection handler has not been implemented yet)");
 $sheet->set_cell (10, 1, 'left', "You can add buttons, charts, pixmaps, and other widgets");

 $sheet->signal_connect (key_press_event => \&clipboard_handler);
 $sheet->signal_connect (resize_range => \&resize_handler);
 $sheet->signal_connect (move_range => \&move_handler);
 $sheet->signal_connect (changed => \&alarm_change);
 $sheet->signal_connect (activate => \&alarm_activate);
 $sheet->signal_connect (deactivate => \&alarm_deactivate);
 $sheet->signal_connect (traverse => \&alarm_traverse);

 $curve =  Gtk2::Curve->new;
 $curve->set_range (0, 200, 0, 200);
 $curve->show;

 my ($pixmap, $mask) = Gtk2::Gdk::Pixmap->colormap_create_from_xpm_d (undef, $colormap, undef, @bullet_xpm);

 my $bullet;
 foreach my $i (4..10) {
      $bullet = Gtk2::Pixmap->new ($pixmap, $mask);
      $bullet->show;
      $sheet->attach ($bullet, $i, 0, 'expand', 'expand', 0, 0);
 }

 ($pixmap, $mask) = Gtk2::Gdk::Pixmap->colormap_create_from_xpm_d (undef, $colormap, undef, @smile_xpm);
 my $smile = Gtk2::Pixmap->new ($pixmap, $mask);
 $smile->show;
 $sheet->button_attach ($smile, -1, 5);


 my $show_button = Gtk2::Button->new ("Show me a plot");
 $show_button->show;
 $show_button->set_usize (100,60);
# gtk_sheet_get_cell_area(GTK_SHEET(sheets[0]), 12, 2, &area);
# gtk_sheet_put(GTK_SHEET(sheets[0]), show_button, area.x, area.y);
 $sheet->attach ($show_button, 12, 2, 'fill', 'fill', 5, 5);

 $show_button->signal_connect (clicked => \&show_child);
# gtk_signal_connect(GTK_OBJECT(sheet),
#                    "button_press_event",
#                    (GtkSignalFunc) do_popup, 
#                    NULL);
}


sub build_example2 {
 my $sheet = shift;
 my $color;
 my $b;
 
 $sheet->set_autoscroll (TRUE);
# gtk_sheet_set_justify_entry(sheet, TRUE);
 $sheet->set_selection_mode ('single');

 my $range = Gtk2::Ex::Sheet::Range->new (0, 0, 2, $sheet->maxcol);
 $sheet->range_set_editable ($range, FALSE);
 $color = Gtk2::Gdk::Color->parse ("light gray");
 $sheet->get_colormap->color_alloc ($color);
 $sheet->range_set_background(sheet, $range, &color);
 $color = Gtk2::Gdk::Color->parse ("blue");
 $sheet->get_colormap->color_alloc ($color);
 $sheet->range_set_foreground(sheet, $range, &color);
 $range->row0 (1);
 $color = Gtk2::Gdk::Color->parse ("red");
 $sheet->get_colormap->color_alloc ($color);
 $sheet->range_set_foreground(sheet, $range, &color);
 $range->row0 (2);
 $color = Gtk2::Gdk::Color->parse ("black");
 $sheet->get_colormap->color_alloc ($color);
 $sheet->range_set_foreground ($range, &color);

# gtk_sheet_row_set_sensitivity(sheet, 0, FALSE);
# gtk_sheet_row_set_sensitivity(sheet, 1, FALSE);
# gtk_sheet_row_set_sensitivity(sheet, 2, FALSE);
 $sheet->set_cell (0, 2, 'center', "Click the right mouse button to display a popup");
 $sheet->set_cell (1, 2, 'center', "You can connect a parser to the 'set cell' signal");
 $sheet->set_cell (2, 2, 'center', "(Try typing numbers)");
 $sheet->set_active_cell (3, 0);

# gtk_sheet_set_update_policy(sheet, GTK_UPDATE_CONTINUOUS, GTK_UPDATE_CONTINUOUS);

 $sheet->signal_connect (button_press_event => \&do_popup);
# $sheet->signal_connect (set_cell => \&parse_numbers);
 $sheet->signal_connect (activate => \&alarm_activate);
 $sheet->signal_connect (deactivate => \&alarm_deactivate);
 $sheet->signal_connect (traverse => \&alarm_traverse);

 $sheet->set_row_height (12, 60);
 $b = Gtk2::Button->new ("GTK_FILL");
 $sheet->attach ($b, 12, 2, 'fill', 'fill', 5, 5);
 $b->show;
 $b = Gtk2::Button->new ("GTK_EXPAND");
 $sheet->attach ($b, 12, 3, 'expand', 'expand', 5, 5);
 $b->show;
 $b = Gtk2::Button->new ("GTK_SHRINK");
 $sheet->attach ($b, 12, 4, 'shrink', 'shrink', 5, 5);
 $b->show;
}

sub build_example3 {
 my $sheet = shift;
 my $color;
 
 $sheet->show_grid (FALSE);

 my $range = Gtk2::Ex::Sheet::Range->new (0, 0, 10, 6);
 $color = Gtk2::Gdk::Color->parse ("orange");
 $sheet->get_colormap->color_alloc ($color);
 $sheet->range_set_background ($range, $color);
 $color = Gtk2::Gdk::Color->parse ("violet");
 $sheet->get_colormap->color_alloc ($color);
 $sheet->range_set_foreground ($range, $color);
 $range->row0 (1);
 $color = Gtk2::Gdk::Color->parse ("blue");
 $sheet->get_colormap->color_alloc ($color);
 $sheet->range_set_background ($range, $color);
 $range->coli (0);
 $color = Gtk2::Gdk::Color->parse ("dark green");
 $sheet->get_colormap->color_alloc ($color);
 $sheet->range_set_background ($range, $color);

 $range->row0 (0); 
 $color = Gtk2::Gdk::Color->parse ("dark blue");
 $sheet->get_colormap->color_alloc ($color);
 $sheet->range_set_border_color ($range, $color);
 $sheet->range_set_border ($range, 'GTK_SHEET_RIGHT_BORDER', 4, 1);
 $range->coli (0);
 $range->col0 (0);
 $range->rowi (0);
 $color = Gtk2::Gdk::Color->parse ("red");
 $sheet->get_colormap->color_alloc ($color);
 $sheet->range_set_background ($range, $color);
 $sheet->range_set_border ($range, ['GTK_SHEET_RIGHT_BORDER',
                                    'GTK_SHEET_BOTTOM_BORDER'], 4, 0);
 $range->rowi (0);
 $range->col0 (1);
 $range->coli (6);
 $color = Gtk2::Gdk::Color->parse ("dark blue");
 $sheet->get_colormap->color_alloc ($color);
 $sheet->range_set_border_color ($range, $color);
 $sheet->range_set_border ($range, 'GTK_SHEET_BOTTOM_BORDER', 4, 1);

 $sheet->set_autoresize (TRUE);

 $sheet->change_entry ('Gtk2::Combo');

 $sheet->signal_connect (traverse => \&change_entry);
}

sub set_cell {
  my ($widget, $insert, $text_legth, $position) = @_;

  my $sheet_entry = $widget->get_entry;
  my $text = $sheet_entry->get_text;
  $entry->set_text ($text) if defined $text;
  $entry->unset_flags ('has-focus');
  $sheet_entry->set_flags ('has-focus');
} 

sub show_sheet_entry {
 my ($widget) = @_;

 return unless $widget->has_focus;
 my $sheet_entry = current_sheet()->get_entry;
 $sheet_entry->set_text ($entry->get_text);
}

sub activate_sheet_entry {
  my ($widget) = @_;

  my $justification = 'GTK_JUSTIFY_LEFT';
  
  my $sheet = current_sheet();
  my $row = $sheet->active_cell->row;
  my $col = $sheet->active_cell->col;

  my $sheet_entry = $sheet->get_entry;

  $justification = $sheet_entry->justification
    if $sheet_entry->isa ('Gtk2::Ex::Sheet::ItemEntry');

  $sheet->set_cell ($row, $col, $justification, $sheet_entry->get_text);

}

sub show_entry {
 my ($widget) = @_;
 return unless $widget->has_focus;
 my $sheet_entry = current_sheet()->get_entry;
 $entry->set_text ($sheet_entry->get_text);
}

sub justify {
  my $justification = shift;
  $left_button->set_active   ($justification eq 'left'   ? 'active' : 'normal');
  $center_button->set_active ($justification eq 'center' ? 'active' : 'normal');
  $right_button->set_active  ($justification eq 'right'  ? 'active' : 'normal');
  my $current = current_sheet ();
  $current->range_set_justification ($current->range, $justification);
}

sub justify_left { justify ('left') }
sub justify_center { justify ('center') }
sub justify_right { justify ('right') }

sub activate_sheet_cell {
  my ($sheet, $row, $column) = @_;
  my $cell;

  my $sheet_entry = $sheet->get_entry;

# XXX
#  if ($sheet->column[column].name) {
#    $cell = sprintf "  %s:%d  ", $sheet->column[column].name, $row;
#  } else {
    $cell = sprintf " ROW: %d COLUMN: %d ", $row, $column;
#  }

  $location->set ($cell);

  $entry->set_max_length ($sheet_entry->text_max_length);
  $entry->set_text ($sheet_entry->get_text || '');

  my $attributes = $sheet->get_attributes ($sheet->active_cell->row,
				           $sheet->active_cell->col);

  $entry->set_editable ($attributes->is_editable);

  justify ($attributes->justification);

  return TRUE;
}

void 
change_border (GtkWidget *widget, gint border)
{
   GtkSheet *current;
   gint cur_page;
   GtkSheetRange range, auxrange;
   gint border_mask, border_width=3;
   gint auxcol, auxrow;
   gint i,j;

   cur_page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
   current=GTK_SHEET(sheets[cur_page]);

   range=current->range;

   gtk_sheet_range_set_border(current, &range, 0, 0, 0);

   switch(border){
     case 0:
       break;
     case 1:
       border_mask = GTK_SHEET_TOP_BORDER;
       range.rowi = range.row0;
       gtk_sheet_range_set_border(current, &range, border_mask, border_width, 0);
       break;
     case 2:
       border_mask = GTK_SHEET_BOTTOM_BORDER;
       range.row0 = range.rowi;
       gtk_sheet_range_set_border(current, &range, border_mask, border_width, 0);
       break;
     case 3:
       border_mask = GTK_SHEET_RIGHT_BORDER;
       range.col0 = range.coli;
       gtk_sheet_range_set_border(current, &range, border_mask, border_width, 0);
       break;
     case 4:
       border_mask = GTK_SHEET_LEFT_BORDER;
       range.coli = range.col0;
       gtk_sheet_range_set_border(current, &range, border_mask, border_width, 0);
       break;
     case 5:
       if(range.col0 == range.coli){
	   border_mask = GTK_SHEET_LEFT_BORDER | GTK_SHEET_RIGHT_BORDER;
	   gtk_sheet_range_set_border(current, &range, border_mask, border_width, 0);
           break;
	}
       border_mask = GTK_SHEET_LEFT_BORDER;
       auxcol=range.coli;
       range.coli = range.col0;
       gtk_sheet_range_set_border(current, &range, border_mask, border_width, 0);
       border_mask = GTK_SHEET_RIGHT_BORDER;
       range.col0 = range.coli=auxcol; 
       gtk_sheet_range_set_border(current, &range, border_mask, border_width, 0);
       break;
     case 6:
       if(range.row0 == range.rowi){
	   border_mask = GTK_SHEET_TOP_BORDER | GTK_SHEET_BOTTOM_BORDER;
	   gtk_sheet_range_set_border(current, &range, border_mask, border_width, 0);
           break;
	}
       border_mask = GTK_SHEET_TOP_BORDER;
       auxrow=range.rowi;
       range.rowi = range.row0;
       gtk_sheet_range_set_border(current, &range, border_mask, border_width, 0);
       border_mask = GTK_SHEET_BOTTOM_BORDER;
       range.row0=range.rowi=auxrow; 
       gtk_sheet_range_set_border(current, &range, border_mask, border_width, 0);
       break;
     case 7:
       border_mask = GTK_SHEET_RIGHT_BORDER | GTK_SHEET_LEFT_BORDER;
       gtk_sheet_range_set_border(current, &range, border_mask, border_width, 0);
       break;
     case 8:
       border_mask = GTK_SHEET_BOTTOM_BORDER | GTK_SHEET_TOP_BORDER;
       gtk_sheet_range_set_border(current, &range, border_mask, border_width, 0);
       break;
     case 9:
       gtk_sheet_range_set_border(current, &range, 15, border_width, 0);
       for(i=range.row0; i<=range.rowi; i++)
        for(j=range.col0; j<=range.coli; j++){
          border_mask = 15;
          auxrange.row0 = i;
          auxrange.rowi = i;
          auxrange.col0 = j;
          auxrange.coli = j;
          if(i == range.rowi) border_mask ^= GTK_SHEET_BOTTOM_BORDER;
          if(i == range.row0) border_mask ^= GTK_SHEET_TOP_BORDER;
          if(j == range.coli) border_mask ^= GTK_SHEET_RIGHT_BORDER;
          if(j == range.col0) border_mask ^= GTK_SHEET_LEFT_BORDER;
          if(border_mask != 15)
            gtk_sheet_range_set_border(current, &auxrange, border_mask, 
                                       border_width, 0);
       }

       break;
     case 10:
       for(i=range.row0; i<=range.rowi; i++)
        for(j=range.col0; j<=range.coli; j++){
          border_mask = 0;
          auxrange.row0 = i;
          auxrange.rowi = i;
          auxrange.col0 = j;
          auxrange.coli = j;
          if(i == range.rowi) border_mask |= GTK_SHEET_BOTTOM_BORDER;
          if(i == range.row0) border_mask |= GTK_SHEET_TOP_BORDER;
          if(j == range.coli) border_mask |= GTK_SHEET_RIGHT_BORDER;
          if(j == range.col0) border_mask |= GTK_SHEET_LEFT_BORDER;
          if(border_mask != 0)
            gtk_sheet_range_set_border(current, &auxrange, border_mask, 
                                       border_width, 0);
       }

       break;
     case 11:
       border_mask = 15;
       gtk_sheet_range_set_border(current, &range, border_mask, border_width, 0);
       break;


   }

}


void 
change_fg(GtkWidget *widget, gint i, GdkColor *color)
{
   GtkSheet *current;
   gint cur_page;
   GdkGC *tmp_gc;

   cur_page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
   current=GTK_SHEET(sheets[cur_page]);

   gtk_sheet_range_set_foreground(current, &current->range, color);

   tmp_gc=gdk_gc_new(window->window);
   gdk_gc_set_foreground(tmp_gc,color);
   gdk_draw_rectangle(GTK_PIXMAP(fg_pixmap)->pixmap,
		      tmp_gc, TRUE, 5,20,16,4);
   gtk_widget_queue_draw(fg_pixmap); 
   gdk_gc_unref(tmp_gc);
}

void 
change_bg(GtkWidget *widget, gint i, GdkColor *color)
{
   GtkSheet *current;
   gint cur_page;
   GdkGC *tmp_gc;

   cur_page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
   current=GTK_SHEET(sheets[cur_page]);

   gtk_sheet_range_set_background(current, &current->range, color);

   tmp_gc=gdk_gc_new(window->window);
   gdk_gc_set_foreground(tmp_gc,color);
   gdk_draw_rectangle(GTK_PIXMAP(bg_pixmap)->pixmap,
		      tmp_gc, TRUE, 4,20,18,4);
   gtk_widget_draw(bg_pixmap, NULL); 
   gdk_gc_unref(tmp_gc);
}

sub do_hide_row_titles {
  current_sheet()->hide_row_titles;
}

sub do_hide_column_titles {
  current_sheet()->hide_column_titles;
}

sub do_show_row_titles {
  current_sheet()->show_row_titles;
}

sub do_show_column_titles {
  current_sheet()->show_column_titles;
}

##if 0
#void
#new_font(GtkFontCombo *font_combo, gpointer data)
#{
#  GtkSheet *current; 
#  gint cur_page;
#
#  cur_page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
#  current=GTK_SHEET(sheets[cur_page]);
#
#  gtk_sheet_range_set_font(current, &current->range, gtk_font_combo_get_font_description(font_combo));
#}
##endif

#int main(int argc, char *argv[]) 
{
  my @title (
  	"Example 1",
	"Example 2", 
	"Example 3", 
	"Example 4"
  );
  my @folder = (
	"Folder 1",
	"Folder 2", 
	"Folder 3", 
	"Folder 4"
  );

  colormap = gdk_colormap_get_system();

  $window = Gtk2::Window->new;
  $window->set_title ("GtkSheet Demo");
  $window->set_default_size (900, 600);

  $window->signal_connect (destroy => sub {Gtk2->main_quit});

  my $main_vbox = Gtk2::VBox->new (FALSE, 1);
  $main_vbox->set_border_width (0); 
  $window->add ($main_vbox);
  $main_vbox->show;

  my $show_hide_box = Gtk2::HBox->new;
  my $hide_row_titles = Gtk2::Button->new ("Hide Row Titles"); 
  my $hide_column_titles = Gtk2::Button->new ("Hide Column Titles"); 
  my $show_row_titles = Gtk2::Button->new ("Show Row Titles"); 
  my $show_column_titles = Gtk2::Button->new ("Show Column Titles");

  $show_hide_box->add ($hide_row_titles); 
  $show_hide_box->add ($hide_column_titles);
  $show_hide_box->add ($show_row_titles);
  $show_hide_box->add ($show_column_titles);

  $hide_row_titles->show;
  $hide_column_titles->show;
  $show_row_titles->show;
  $show_column_titles->show;

  $hide_row_titles->signal_connect (clicked => \&do_hide_row_titles);
  $hide_column_titles->signal_connect (clicked => \&do_hide_column_titles);
  $show_row_titles->signal_connect (clicked => \&do_show_row_titles);
  $show_column_titles->signal_connect (clicked => \&do_show_column_titles);

  $main_vbox->pack_start ($show_hide_box, FALSE, TRUE, 0);
  $show_hide_box->show;

  my $toolbar = Gtk2::Toolbar->new;;

  $toolbar->append_space;

##if 0
#  font_combo = gtk_font_combo_new();
#  gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
#			    font_combo, "font", "font");
#
#  gtk_widget_set_usize(GTK_FONT_COMBO(font_combo)->italic_button, 32, 32);
#  gtk_widget_set_usize(GTK_FONT_COMBO(font_combo)->bold_button, 32, 32);
#  $font_combo->show;
#  gtk_signal_connect(GTK_OBJECT(font_combo), "changed", GTK_SIGNAL_FUNC(new_font), NULL);
##endif

  $toolbar->append_space;

  $left_button = Gtk2::ToggleButton->new();
  $toolbar->append_widget ($left_button, "justify left", "justify left");
  $left_button->show;

  $left_button->signal_connect (released => \&justify_left);

  $center_button = Gtk2::ToggleButton->new();
  $toolbar->append_widget ($center_button, "justify center", "justify center");
  $center_button->show;

  $center_button->signal_connect (released => \&justify_center);


  $right_button = Gtk2::ToggleButton->new();
  $toolbar->append_widget ($right_button, "justify right", "justify right");
  $right_button->show;

  $right_button->signal_connect (released => sub \&justify_right);

  $toolbar->append_space;

##if 0
#  bordercombo=gtk_border_combo_new();
#  gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
#			    bordercombo, "border", "border");
#  gtk_widget_set_usize(GTK_COMBO_BUTTON(bordercombo)->button, 32, 32);
#  $bordercombo->show;
#
#  gtk_signal_connect(GTK_OBJECT(bordercombo),
#		     "changed", (GtkSignalFunc)change_border, NULL);
##endif

  $toolbar->append_space;

##if 0
#  fgcolorcombo=gtk_color_combo_new();
#  gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
#			    fgcolorcombo, "font color", "font color");
#  $fgcolorcombo->show;
#
#  gtk_signal_connect(GTK_OBJECT(fgcolorcombo),
#		     "changed", (GtkSignalFunc)change_fg, NULL);
##endif

  $toolbar->append_space;

##if 0
#  bgcolorcombo=gtk_color_combo_new();
#  gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
#			    bgcolorcombo, "background color", "background color");
#  $bgcolorcombo->show;
#
#  gtk_signal_connect(GTK_OBJECT(bgcolorcombo),
#		     "changed", (GtkSignalFunc)change_bg, NULL);
##endif

  $toolbar->append_space;

##if 0
#  toggle_combo = gtk_toggle_combo_new(5, 5);
#  gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
#			    toggle_combo, "test", "test");
#  gtk_widget_set_usize(GTK_COMBO_BUTTON(toggle_combo)->button, 32, 32);
#  $toggle_combo->show;
##endif

  $main_vbox->pack_start ($toolbar, FALSE, TRUE, 0);
  $toolbar->show;

  my $status_box = Gtk2::HBox->new (FALSE, 1);
  $status_box->set_border_width (0);
  $main_vbox->pack_start ($status_box, FALSE, TRUE, 0);
  $status_box->show;

  $location = Gtk2::Label->new; 
  my $requisition = $location->size_request; 
  $location->set_usize (160, request.height); # XXX ?
  $status_box->pack_start ($location, FALSE, TRUE, 0);
  $location->show;

  $entry = Gtk2::Entry->new;
  $status_box->add ($entry); 
  $entry->show;

  $notebook = Gtk2::Notebook->new;
  $notebook->set_tab_pos ('bottom');
  $main_vbox->add ($notebook);
  $notebook->show;

  for ($i = 0; $i < NUM_SHEETS; $i++) {
   $sheets[$i] = Gtk2::Ex::Sheet->new (1000, 26, $title[$i]);

   my $scrolled_window = Gtk2::ScrolledWindow->new;
   $scrolled_window->add ($sheets[$i]);
   $sheets[$i]->show;

   $scrolled_window->show;

   my $label = Gtk2::Label->new (@folder[$i]);
   $notebook->append_page ($scrolled_window, $label);

   $sheets[$i]->get_entry->signal_connect (changed => \&show_entry);
   $sheets[$i]->signal_connect (activate => \&activate_sheet_cell);
  }


  $entry->signal_connect (changed => \&show_sheet_entry);
  $entry->signal_connect (activate => \&activate_sheet_entry);


  build_example1 ($sheets[0]); 
  build_example2 ($sheets[1]);
  build_example3 ($sheets[2]);

  $sheets[2]->get_entry->signal_connect (changed => \&show_entry);


  pixmap=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
				    left_just);
  my $tpixmap = Gtk2::Pixmap->new ($pixmap, $mask);
  $left_button->add ($tpixmap);
  $tpixmap->show;

  pixmap=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
				    center_just);
  $tpixmap = Gtk2::Pixmap->new ($pixmap, $mask);
  $center_button->add ($tpixmap);
  $tpixmap->show;

  pixmap=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
				    right_just);
  $tpixmap = Gtk2::Pixmap->new ($pixmap, $mask);
  $right_button->add ($tpixmap);
  $tpixmap->show;

##if 0
#  pixmap=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
#				    paint);
#
#  bg_pixmap=Gtk2::Pixmap->new (pixmap, mask);
#  gtk_container_add(GTK_CONTAINER(GTK_COMBO_BUTTON(bgcolorcombo)->button),
#		    bg_pixmap);
#
#  $bg_pixmap->show;
#  gdk_pixmap_unref(pixmap);
#  gdk_bitmap_unref(mask);
#
#  pixmap=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
#				    font);
#
#
#  fg_pixmap=Gtk2::Pixmap->new (pixmap, mask);
#  gtk_container_add(GTK_CONTAINER(GTK_COMBO_BUTTON(fgcolorcombo)->button),
#		    fg_pixmap);
#
#  $fg_pixmap->show;
#  gdk_pixmap_unref(pixmap);
#  gdk_bitmap_unref(mask);
##endif

  $window->show;

  Gtk2->main;

  return(0);
}


