#!/usr/bin/perl -w

use strict;
use Gtk2::TestHelper tests => 57;
use Gtk2::Ex::Sheet;


my $sheet = Gtk2::Ex::Sheet->new (25, 25, "testing");
isa_ok ($sheet, 'Gtk2::Ex::Sheet');
isa_ok ($sheet, 'Gtk2::Widget');
is ($sheet->maxrow, 25-1);
is ($sheet->maxcol, 25-1);


###/* create a new browser sheet. It cells can not be edited */
#GtkWidget * gtk_sheet_new_browser (class, guint rows, guint columns, const gchar *title);
#    C_ARGS:
#	rows, columns, title


my $vadj = $sheet->get_vadjustment;
my $hadj = $sheet->get_hadjustment;
ok (not defined $vadj);
ok (not defined $hadj);
$sheet->set_hadjustment ($vadj); 
$sheet->set_vadjustment ($hadj); 

my $scolled_window = Gtk2::ScrolledWindow->new;
$sheet->set_hadjustment ($scolled_window->get_hadjustment); 
$sheet->set_vadjustment ($scolled_window->get_vadjustment); 
$vadj = $sheet->get_vadjustment;
$hadj = $sheet->get_hadjustment;
isa_ok ($vadj, 'Gtk2::Adjustment');
isa_ok ($hadj, 'Gtk2::Adjustment');

## Change entry
$sheet->change_entry ('Gtk2::SpinButton');

##/* Returns sheet's entry widget */

my $entry = $sheet->get_entry;

###GtkWidget * $sheet->get_entry_widget;

my $state = $sheet->get_state;
print "state $state\n";

is ($sheet->get_columns_count, 25, 'get columns count');
is ($sheet->get_rows_count, 25, 'get rows count');

my $range = $sheet->get_visible_range;
isa_ok ($range, 'Gtk2::Ex::Sheet::Range');
$sheet->set_selection_mode ('single');

# some booleans
foreach my $prop (qw(autoresize autoscroll clip_text justify_entry locked)) {
	my $setter = "set_$prop";
	$sheet->$setter (TRUE);
	ok ($sheet->$prop, "$prop");
	$sheet->$setter (FALSE);
	ok (!$sheet->$prop, "!$prop");
}

$sheet->set_title ('this is a title for the sheet');

$sheet->freeze;
$sheet->thaw;

##/* Background colors */
my $back_color = Gtk2::Gdk::Color->parse ('white');
my $grid_color = Gtk2::Gdk::Color->parse ('purple');
$sheet->set_background ($back_color);
$sheet->set_grid ($grid_color);

$sheet->set_background (undef);
$sheet->set_grid (undef);

$sheet->show_grid (FALSE);
ok (!$sheet->grid_visible, '! grid visible');
$sheet->show_grid (TRUE);
ok ($sheet->grid_visible, 'grid visible');

$sheet->set_column_title (1, "Poobah");
is ($sheet->get_column_title (1), "Poobah", "column title");
$sheet->set_row_title (1, "Pompitus");
is ($sheet->get_row_title (1), "Pompitus", "row title");

$sheet->row_button_add_label (2, "whee");
is ($sheet->row_button_get_label (2), "whee", "row button label");
$sheet->column_button_add_label (2, "w00t");
is ($sheet->column_button_get_label (2), "w00t", "column button label");

$sheet->row_button_justify (15, 'center');
$sheet->column_button_justify (14, 'right');

$sheet->moveto (15, 14, 0.25, 0.75);

$sheet->set_row_titles_width (75);
$sheet->set_column_titles_height (30);

$sheet->hide_column_titles;
ok (!$sheet->column_titles_visible);
$sheet->show_column_titles;
ok ($sheet->column_titles_visible);

$sheet->hide_row_titles;
ok (!$sheet->row_titles_visible);
$sheet->show_row_titles;
ok ($sheet->row_titles_visible);


$sheet->column_set_sensitivity (8, FALSE);
$sheet->columns_set_sensitivity (TRUE);
$sheet->columns_set_resizable (FALSE);
ok (!$sheet->columns_resizable);
$sheet->columns_set_resizable (TRUE);
ok ($sheet->columns_resizable);


$sheet->row_set_sensitivity (8, FALSE);
$sheet->rows_set_sensitivity (TRUE);
$sheet->rows_set_resizable (FALSE);
ok (!$sheet->rows_resizable);
$sheet->rows_set_resizable (TRUE);
ok ($sheet->rows_resizable);

$sheet->column_set_visibility (9, FALSE);
$sheet->column_label_set_visibility (10, FALSE);
$sheet->columns_labels_set_visibility (TRUE);

$sheet->row_set_visibility (9, FALSE);
$sheet->row_label_set_visibility (10, FALSE);
$sheet->rows_labels_set_visibility (TRUE);

$sheet->select_row (3);
$sheet->select_column (5);

##/* save selected range to "clipboard" */
$sheet->clip_range (Gtk2::Ex::Sheet::Range->new (1,1, 5,5));
ok ($sheet->in_clip);

$sheet->unclip_range;
ok (!$sheet->in_clip);


# must be realized and parented before this works
my $window = Gtk2::Window->new;
$scolled_window->add ($sheet);
$window->add ($scolled_window);
$window->show_all;
$window->show_now;
$sheet->select_range (Gtk2::Ex::Sheet::Range->new (1,1, 5,5)); 
$sheet->unselect_range;

ok (! $sheet->set_active_cell (50, 50), 'can\'t activate non-existent row');
ok ($sheet->set_active_cell (5, 5), 'set_active_cell');

my ($row, $column) = $sheet->get_active_cell;
is ($row, 5, 'right row');
is ($column, 5, 'right column');

##/* set cell contents and allocate memory if needed */
$sheet->set_cell (5, 10, 'center', 'this is great!');

$sheet->set_cell_text (1, 3, "this is the song that never ends");
is ($sheet->cell_get_text (1, 3), "this is the song that never ends",
    'cell_get_text');
$sheet->cell_clear (1, 3);
is ($sheet->cell_get_text (1, 3), undef, 'clear');
$sheet->cell_delete (1, 3);

##/* clear range contents. If range==NULL the whole sheet will be cleared */
$sheet->range_clear (undef);
$sheet->range_clear (Gtk2::Ex::Sheet::Range->new (1, 1, 5, 5));

##/* clear range contents and remove links */
$sheet->range_delete (Gtk2::Ex::Sheet::Range->new (1, 1, 5, 5));
$sheet->range_delete (undef);

##/* get cell state: GTK_STATE_NORMAL, GTK_STATE_SELECTED */
is ('normal', $sheet->cell_get_state (1, 1));
$sheet->select_row (1);
is ('selected', $sheet->cell_get_state (1, 1));

### Handles cell links
#$sheet->link_cell (gint row, gint col, gpointer link);
#gpointer $sheet->get_link (gint row, gint col);
#$sheet->remove_link (gint row, gint col);

##/* get row and column correspondig to the given position in the screen */
## gboolean $sheet->get_pixel_info (gint x, gint y, gint * row, gint * column);
($row, $column) = $sheet->get_pixel_info (25, 25);
ok (defined $row);
ok (defined $column);

##/* get area of a given cell */
my $rect = $sheet->get_cell_area (50, 100);
ok (! defined $rect);

$sheet->set_column_width (4, 50);
$sheet->set_row_height (2, 25);
$rect = $sheet->get_cell_area (2, 4);
isa_ok ($rect, 'Gtk2::Gdk::Rectangle');
is ($rect->width, 50);
is ($rect->height, 25);

##/* append ncols columns to the end of the sheet */
$sheet->add_column (5);
is ($sheet->maxcol, 25-1+5);

##/* append nrows row to the end of the sheet */
$sheet->add_row (5);
is ($sheet->maxrow, 25-1+5);

##/* insert nrows rows before the given row and pull right */
$sheet->insert_rows (10, 5);
is ($sheet->maxrow, 25-1+5+5);

##/* insert ncols columns before the given col and pull down */
$sheet->insert_columns (10, 5);
is ($sheet->maxcol, 25-1+5+5);

##/* delete nrows rows starting in row */
$sheet->delete_rows (10, 10);
is ($sheet->maxrow, 25-1);

##/* delete ncols columns starting in col */
$sheet->delete_columns (10, 10);
is ($sheet->maxcol, 25-1);

##/* set abckground color of the given range */
$range = Gtk2::Ex::Sheet::Range->new (5, 5, 10, 10);
my $color = Gtk2::Gdk::Color->parse ('blue');
$sheet->range_set_background ($range, $color);
$sheet->range_set_background (undef, undef);

##/* set foreground color (text color) of the given range */
$sheet->range_set_foreground ($range, $color);
$sheet->range_set_foreground (undef, undef);

##/* set text justification (GTK_JUSTIFY_LEFT, RIGHT, CENTER) of the given range.
## * The default value is GTK_JUSTIFY_LEFT. If autoformat is on, the
## * default justification for numbers is GTK_JUSTIFY_RIGHT */
$sheet->range_set_justification (undef, 'right');
$sheet->range_set_justification ($range, 'center');

$sheet->column_set_justification (4, 'left');

##/* set if cell contents can be edited or not in the given range:
## * accepted values are TRUE or FALSE. */
$sheet->range_set_editable (undef, TRUE);
$sheet->range_set_editable ($range, FALSE);

##/* set if cell contents are visible or not in the given range:
## * accepted values are TRUE or FALSE.*/
$sheet->range_set_visible ($range, FALSE);
$sheet->range_set_visible (undef, TRUE);

##/* set cell border style in the given range.
## * mask values are CELL_LEFT_BORDER, CELL_RIGHT_BORDER, CELL_TOP_BORDER,
## * CELL_BOTTOM_BORDER
## * width is the width of the border line in pixels 
## * line_style is the line_style for the border line */
$sheet->range_set_border (undef, ['bottom', 'right'], 5, 'double-dash');
$sheet->range_set_border ($range, ['top', 'left'], 1, 'solid');

##/* set border color for the given range */
$sheet->range_set_border_color (undef, $color);
$sheet->range_set_border_color ($range, $color);

##/* set font for the given range */
my $font = Gtk2::Pango::FontDescription->from_string ("monospace 10");
$sheet->range_set_font (undef, $font);
$font = Gtk2::Pango::FontDescription->from_string ("sans 10");
$sheet->range_set_font ($sheet->get_visible_range, $font);

##/* get cell attributes of the given cell */
##/* TRUE means that the cell is currently allocated */
#gboolean $sheet->get_attributes (gint row, gint col, GtkSheetCellAttr *attributes);

# XXX not sure how these are supposed to work
#my $button = Gtk2::Button->new ('Click me');
#$button->show;
#$sheet->attach_floating ($button, 10, 5);
#
#$button = Gtk2::Button->new ('and another');
#$button->show;
#$sheet->attach_default ($button, 11, 5);
#
#$button = Gtk2::Button->new ('still anohter');
#$button->show;
#$sheet->attach ($button, 12, 5, 'expand', 'fill', 5, 5);
#
#$sheet->move_child ($button, 100, 50);
#
#my $child = $sheet->put ($button, 50, 100);
#isa_ok ($child, 'Gtk2::Ex::Sheet::Child');
#
#$child = $sheet->get_child_at (11, 5);
#isa_ok ($child, 'Gtk2::Ex::Sheet::Child');
#
#$sheet->button_attach (Gtk2::Label->new ('foo'), 13, 5);


